//
//  TagsGenerator.swift
//  
//
//  Created by Thibaut Schmitt on 10/01/2022.
//

import Foundation
import ToolCore
import CoreVideo

class TagsGenerator {
    static func writeExtensionFiles(sections: [TagSection], target: String, tags: [String], staticVar: Bool, extensionName: String, extensionFilePath: String) {
        // Get target type from enum
        target.split(separator: " ").forEach { target in
            Tags.TargetType.allCases.forEach { value in
                
            }
        }
        
        let targetsString: [String] = target.components(separatedBy: " ")
        var targets: [Tags.TargetType] = []
                
        Tags.TargetType.allCases.forEach { enumTarget in
            if targetsString.contains(enumTarget.value) {
                targets.append(enumTarget)
            }
        }
        
        
        
        // Get extension content
        let extensionFileContent = Self.getExtensionContent(sections: sections,
                                                            targets: targets,
                                                            tags: tags,
                                                            staticVar: staticVar,
                                                            extensionName: extensionName)
        
        // Write content
        let extensionFilePathURL = URL(fileURLWithPath: extensionFilePath)
        do {
            try extensionFileContent.write(to: extensionFilePathURL, atomically: false, encoding: .utf8)
        } catch (let error) {
            let error = StringiumError.writeFile(extensionFilePath, error.localizedDescription)
            print(error.description)
            Stringium.exit(withError: error)
        }
    }
    
    // MARK: - Extension content
    
    static func getExtensionContent(sections: [TagSection], targets: [Tags.TargetType], tags: [String], staticVar: Bool, extensionName: String) -> String {
        [
            Self.getHeader(extensionClassname: extensionName, staticVar: staticVar, targets: targets),
            Self.getProperties(sections: sections, target: "target", tags: tags, staticVar: staticVar),
            Self.getFooter()
        ]
            .joined(separator: "\n")
    }
    
    // MARK: - Extension part
    
    private static func getHeader(extensionClassname: String, staticVar: Bool, targets: [Tags.TargetType]) -> String {
        """
        // Generated by ResgenSwift.\(Tags.toolName) \(ResgenSwiftVersion)
        
        \(staticVar ? "typelias Tags = String\n\n" : "")import UIKit
        
        \(Self.getAnalytics(targets: targets))
        
        extension \(extensionClassname) {
            
            // MARK: - Properties
        
            let managers: [AnalyticsManager] = [\(Self.getAnalyticsProperties(targets: targets))]
        """
    }
    
    private static func getAnalyticsProperties(targets: [Tags.TargetType]) -> String {
        let matomo = "MatomoAnalyticsManager()"
        let firebase = "FirebaseAnalyticsManager()"
        
        var result: [String] = []
        
        if targets.contains(Tags.TargetType.matomo) {
            result.append(matomo)
        }
        
        if targets.contains(Tags.TargetType.firebase) {
            result.append(firebase)
        }
        
        return result.joined(separator: ", ")
    }
    
    private static func getAnalytics(targets: [Tags.TargetType]) -> String {
        let proto = """
        // MARK: - Protocol
        
        protocol AnalyticsManager {
            func logScreen(name: String, path: String)
            func logEvent(name: String)
        }
        """
        
        let matomo = """
        // MARK: - Matomo
        
        class MatomoAnalyticsManager: AnalyticsManager {
            func logScreen(name: String, path: String) {
            
            }
        
            func logEvent(name: String) {
            
            }
        }
        """
        
        let firebase = """
        // MARK: - Firebase
        
        class FirebaseAnalyticsManager: AnalyticsManager {
            func logScreen(name: String, path: String) {
                
            }
        
            func logEvent(name: String) {
            
            }
        }
        """
        
        var result: [String] = [proto]
        
        if targets.contains(Tags.TargetType.matomo) {
            result.append(matomo)
        }
        
        if targets.contains(Tags.TargetType.firebase) {
            result.append(firebase)
        }
        
        return result.joined(separator: "\n")
    }
    
    private static func getProperties(sections: [TagSection], target: String, tags: [String], staticVar: Bool) -> String {
        sections
            .compactMap { section in
                // Check that at least one string will be generated
                guard section.hasOneOrMoreMatchingTags(tags: tags) else {
                    return nil// Go to next section
                }
                
                var res = "\n    // MARK: - \(section.name)"
                section.definitions.forEach { definition in
                    guard definition.hasOneOrMoreMatchingTags(inputTags: tags) == true else {
                        return // Go to next definition
                    }
                    
                    if staticVar {
                        res += "\n\n\(definition.getStaticProperty(forTarget: target))"
                    } else {
                        res += "\n\n\(definition.getProperty(forTarget: target))"
                    }
                }
                return res
            }
            .joined(separator: "\n")
    }
    
    private static func getFooter() -> String {
        """
        }
        
        """
    }
}
