//
//  TagDefinition.swift
//  
//
//  Created by Loris Perret on 05/12/2023.
//

import Foundation

class TagDefinition {
    let title: String
    var path: String = ""
    var name: String = ""
    var type: String = ""
    var tags = [String]()
    var comment: String?
    
    var isValid: Bool {
        title.isEmpty == false &&
        name.isEmpty == false &&
        (type == TagType.screen.value || type == TagType.event.value)
    }
    
    init(title: String) {
        self.title = title
    }
    
    static func match(_ line: String) -> TagDefinition? {
        guard line.range(of: "\\[(.*?)]$", options: .regularExpression, range: nil, locale: nil) != nil else {
            return nil
        }
        
        let definitionTitle = line
            .replacingOccurrences(of: ["[", "]"], with: "")
            .removeLeadingTrailingWhitespace()
        
        return TagDefinition(title: definitionTitle)
    }
    
    func hasOneOrMoreMatchingTags(inputTags: [String]) -> Bool {
        if Set(inputTags).intersection(Set(self.tags)).isEmpty {
            return false
        }
        return true
    }
    
    // MARK: -
    
    private func getStringParameters(input: String) -> (inputParameters: [String], translationArguments: [String])? {
        var methodsParameters = [String]()
        
        let printfPlaceholderRegex = try! NSRegularExpression(pattern: "%(?:\\d+\\$)?[+-]?(?:[ 0]|'.{1})?-?\\d*(?:\\.\\d+)?[blcdeEufFgGosxX@]*")
        printfPlaceholderRegex.enumerateMatches(in: input, options: [], range: NSRange(location: 0, length: input.count)) { match, _, stop in
            guard let match = match else { return }
            
            if let range = Range(match.range, in: input), let last = input[range].last {
                switch last {
                case "d", "u":
                    methodsParameters.append("Int")
                case "f", "F":
                    methodsParameters.append("Double")
                case "@", "s", "c":
                    methodsParameters.append("String")
                case "%":
                    // if you need to print %, you have to add %%
                    break
                default:
                    break
                }
            }
        }
        
        if methodsParameters.isEmpty {
            return nil
        }
        
        var inputParameters = [String]()
        var translationArguments = [String]()
        for (index, paramType) in methodsParameters.enumerated() {
            let paramName = "arg\(index)"
            translationArguments.append(paramName)
            inputParameters.append("\(paramName): \(paramType)")
        }
        
        return (inputParameters: inputParameters, translationArguments: translationArguments)
    }
    
    private func getFuncName() -> String {
        var pascalCaseTitle: String = ""
        name.components(separatedBy: " ").forEach { word in
            pascalCaseTitle.append(contentsOf: word.uppercasedFirst())
        }
        
        return "log\(pascalCaseTitle)"
    }
    
    private func getlogFunction() -> String {
        if type == TagType.screen.value {
            "manager.logScreen(name: name, path: path)"
        } else {
            "manager.logEvent(name: name)"
        }
    }
    
    // MARK: - Raw strings
    
    func getProperty(forTarget target: String) -> String {
        return """
            func \(getFuncName())() {
                managers.forEach { manager in
                    \(getlogFunction())
                }
            }
        """
    }
    
    func getStaticProperty(forTarget target: String) -> String {
//        guard let translation = translations[lang] else {
//            let error = StringiumError.langNotDefined(lang, title, reference != nil)
//            print(error.description)
//            Stringium.exit(withError: error)
//        }
//        
//        return """
//            /// Translation in \(lang) :
//            /// \(translation)
//            static var \(title): String {
//                "\(translation)"
//            }
//        """
        return """
            static func \(getFuncName())() {
                managers.forEach { manager in
                    \(getlogFunction())
                }
            }
        """
    }
}

extension TagDefinition {
    enum TagType {
        case screen
        case event
        
        var value: String {
            switch self {
            case .screen:
                "screen"
            case .event:
                "event"
            }
        }
    }
}
